﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PISOCANCPM_Net;

namespace Listen_Mode
{
    public partial class Form1 : Form
    {
        byte BOARD = 0;
        byte PORT = 1;
        byte BAUD = 0;
        byte NODE = 1;

        byte BlockMode = 1;

        ushort DelayTime = 0;
        ushort ret;
        string errstr;

        byte[] ISADD = new byte[128];

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ushort i;

            Combo_Board.Enabled = true;
            Combo_Baud.Enabled = true;
            Combo_Node.Enabled = false;

            Combo_Board.SelectedIndex = 0;
            Combo_Baud.SelectedIndex = 0;
            Combo_Node.SelectedIndex = 0;

            Btn_Initial.Enabled = true;
            Btn_Shutdown.Enabled = false;
            Btn_AddNode.Enabled = false;
            Btn_LoadEDS.Enabled = false;
            Btn_Remove.Enabled = false;

            List_EntryData.Items.Clear();

            for (i = 0; i < 128; i++)
            {
                ISADD[i] = 0;
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            PISOCANCPM.CPM_ShutdownMaster(BOARD, PORT);
        }

        private void Combo_Com_SelectedIndexChanged(object sender, EventArgs e)
        {
            BOARD = (byte)(Combo_Board.SelectedIndex);
        }

        private void Combo_Baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            BAUD = (byte)(Combo_Baud.SelectedIndex);

            switch (BAUD)
            {
                case 0:
                    DelayTime = 15;
                    break;
                case 1:
                    DelayTime = 7;
                    break;
                case 2:
                    DelayTime = 3;
                    break;
                case 3:
                case 4:
                    DelayTime = 1;
                    break;
                case 5:
                case 6:
                case 7:
                    DelayTime = 0;
                    break;
            }
        }

        private void Combo_Node_SelectedIndexChanged(object sender, EventArgs e)
        {
            NODE = (byte)(Combo_Node.SelectedIndex + 1);

            if (ISADD[NODE] == 1)
            {
                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_LoadEDS.Enabled = false;
                Btn_LoadEDS.Visible = false;
                Btn_Remove.Enabled = true;

                UpdateList();
            }
            else if (ISADD[NODE] == 0)
            {
                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_LoadEDS.Enabled = true;
                Btn_LoadEDS.Visible = true;
                Btn_Remove.Enabled = false;

                List_EntryData.Items.Clear();
            }
        }

        private void Btn_Initial_Click(object sender, EventArgs e)
        {
            ret = PISOCANCPM.CPM_InitMaster(BOARD, PORT, 0, BAUD, BlockMode);
            if (ret != 0)
            {
                errstr = "Initial Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Board.Enabled = false;
                Combo_Baud.Enabled = false;
                Combo_Node.Enabled = true;

                Btn_Initial.Enabled = false;
                Btn_Initial.Visible = false;
                Btn_Shutdown.Enabled = true;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_LoadEDS.Enabled = true;
                Btn_LoadEDS.Visible = true;

                // Set to listen mode
                PISOCANCPM.CPM_SetMasterMode(BOARD, PORT, 1, BlockMode);

                Timer1.Enabled = true;
            }
        }

        private void Btn_Shutdown_Click(object sender, EventArgs e)
        {
            ushort i;

            ret = PISOCANCPM.CPM_ShutdownMaster(BOARD, PORT);
            if (ret != 0)
            {
                errstr = "Shutdown Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                Combo_Board.Enabled = true;
                Combo_Baud.Enabled = true;
                Combo_Node.Enabled = false;

                Combo_Baud.SelectedIndex = 0;

                Btn_Initial.Enabled = true;
                Btn_Initial.Visible = true;
                Btn_Shutdown.Enabled = false;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = true;
                Btn_LoadEDS.Enabled = false;
                Btn_LoadEDS.Visible = true;
                Btn_Remove.Enabled = false;

                for (i = 0; i < 128; i++)
                {
                    ISADD[i] = 0;
                }

                List_EntryData.Items.Clear();

                Timer1.Enabled = false;
            }
        }

        private void Btn_AddNode_Click(object sender, EventArgs e)
        {
            ret = PISOCANCPM.CPM_AddNode(BOARD, PORT, NODE, 2, DelayTime, 200, BlockMode);
            if (ret != 0)
            {
                errstr = "AddNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                byte EntryUse;
                ushort PDOId, i, j;
                uint[] EntryData = new uint[8];

                ISADD[NODE] = 1;

                Btn_AddNode.Enabled = false;
                Btn_AddNode.Visible = false;
                Btn_LoadEDS.Enabled = false;
                Btn_LoadEDS.Visible = false;
                Btn_Remove.Enabled = true;

                // Set EMCY ID to master list
                PISOCANCPM.CPM_SetEMCY_List(BOARD, PORT, NODE, (ushort)(0x80 + NODE), BlockMode);

                // Install RxPDO to master PDO list
                for (i = 0; i < 4; i++)
                {
                    PDOId = (ushort)(0x200 + NODE + (0x100 * i));

                    if (i == 0)
                    {
                        EntryUse = 8;
                    }
                    else
                    {
                        EntryUse = 4;
                    }

                    for (j = 0; j < EntryUse; j++)
                    {
                        if (i == 0)
                        {
                            EntryData[j] = (uint)(0x62000108 + (0x100 * j));
                        }
                        else
                        {
                            EntryData[j] = (uint)(0x64110110 + (0x100 * (j + ((i - 1) * EntryUse))));
                        }
                    }

                    PISOCANCPM.CPM_InstallPDO_List(BOARD, PORT, NODE, PDOId, 0, (ushort)(i + 1), (byte)255, 0, EntryUse, EntryData, BlockMode);
                }

                // Install TxPDO to master PDO list
                for (i = 0; i < 4; i++)
                {
                    PDOId = (ushort)(0x180 + NODE + (0x100 * i));

                    if (i == 0)
                    {
                        EntryUse = 8;
                    }
                    else
                    {
                        EntryUse = 4;
                    }

                    for (j = 0; j < EntryUse; j++)
                    {
                        if (i == 0)
                        {
                            EntryData[j] = (ushort)(0x60000108 + (0x100 * j));
                        }
                        else
                        {
                            EntryData[j] = (ushort)(0x64010110 + (0x100 * (j + ((i - 1) * EntryUse))));
                        }
                    }

                    PISOCANCPM.CPM_InstallPDO_List(BOARD, PORT, NODE, PDOId, 1, (ushort)(i + 1), (byte)255, 0, EntryUse, EntryData, BlockMode);
                }
            }
        }

        private void Btn_LoadEDS_Click(object sender, EventArgs e)
        {
            OpenFileDialog EDSFile = new OpenFileDialog();

            EDSFile.Filter = "EDS files (*.eds)|*.eds";
            EDSFile.FilterIndex = 1;

            if (EDSFile.ShowDialog() == DialogResult.OK)
            {
                ret = PISOCANCPM.CPM_EDS_Load(BOARD, PORT, NODE, EDSFile.FileName, DelayTime, 200, BlockMode);
                if (ret != 0)
                {
                    MessageBox.Show("EDS File Load Error = " + ret.ToString("D"));
                    return;
                }
                else
                {
                    ISADD[NODE] = 1;

                    Btn_AddNode.Enabled = false;
                    Btn_AddNode.Visible = false;
                    Btn_LoadEDS.Enabled = false;
                    Btn_LoadEDS.Visible = false;
                    Btn_Remove.Enabled = true;
                }
            }
        }

        private void Btn_Remove_Click(object sender, EventArgs e)
        {
            ret = PISOCANCPM.CPM_RemoveNode(BOARD, PORT, NODE, BlockMode);
            if (ret != 0)
            {
                errstr = "RemoveNode Error, = " + ret.ToString("D");
                MessageBox.Show(errstr);
            }
            else
            {
                ISADD[NODE] = 0;

                Btn_AddNode.Enabled = true;
                Btn_AddNode.Visible = true;
                Btn_LoadEDS.Enabled = true;
                Btn_LoadEDS.Visible = true;
                Btn_Remove.Enabled = false;

                List_EntryData.Items.Clear();
            }
        }

        private void UpdateList()
        {
            byte PDO_Cnt, NMT_State;
            ushort EMCYId, i, j;
            ushort[] Id_List = new ushort[10];
            string Temp_Str;

            List_EntryData.Items.Clear();

            ret = PISOCANCPM.CPM_NMTGetState(BOARD, PORT, NODE, out NMT_State, BlockMode);
            if (ret == 0)
            {
                Temp_Str = "NMT State:";

                switch (NMT_State)
                {
                    case 0x04:  // Stop mode
                        Temp_Str = Temp_Str + " Stop mode";
                        break;
                    case 0x05:  // Operation mode
                        Temp_Str = Temp_Str + " Operation mode";
                        break;
                    case 0x07F: // Pre-operation mode
                        Temp_Str = Temp_Str + " Pre-operation mode";
                        break;
                }
                List_EntryData.Items.Add(Temp_Str);
            }

            ret = PISOCANCPM.CPM_GetEMCYID(BOARD, PORT, NODE, out EMCYId, BlockMode);
            if (ret == 0)
            {
                if (EMCYId != 0)
                {
                    byte IsNew;
                    byte[] Data = new byte[8];

                    ret = PISOCANCPM.CPM_ReadLastEMCY(BOARD, PORT, NODE, out IsNew, Data, BlockMode);
                    if ((ret == 0) || (ret == PISOCANCPM._15_CPM_DataEmpty))
                    {
                        Temp_Str = "EMCY " + EMCYId.ToString("X") + " : ";

                        for (i = 0; i < 8; i++)
                        {
                            Temp_Str = Temp_Str + Data[i].ToString("X2") + " ";
                        }

                        List_EntryData.Items.Add(Temp_Str);
                    }
                }
            }

            ret = PISOCANCPM.CPM_GetRxPDOID(BOARD, PORT, NODE, out PDO_Cnt, Id_List, BlockMode);
            if (ret == 0)
            {
                byte IsNew, DLen;
                byte[] Data = new byte[8];

                for (i = 0; i < PDO_Cnt; i++)
                {
                    for (j = 0; j < 8; j++)
                    {
                        Data[j] = 0;
                    }

                    ret = PISOCANCPM.CPM_GetPDOLastData(BOARD, PORT, Id_List[i], out IsNew, out DLen, Data, BlockMode);
                    if ((ret == 0) || (ret == PISOCANCPM._15_CPM_DataEmpty))
                    {
                        Temp_Str = "RxPDO " + Id_List[i].ToString("X") + " : ";

                        for (j = 0; j < 8; j++)
                        {
                            Temp_Str = Temp_Str + Data[j].ToString("X2") + " ";
                        }

                        List_EntryData.Items.Add(Temp_Str);
                    }
                }
            }

            ret = PISOCANCPM.CPM_GetTxPDOID(BOARD, PORT, NODE, out PDO_Cnt, Id_List, BlockMode);
            if (ret == 0)
            {
                byte IsNew, DLen;
                byte[] Data = new byte[8];

                for (i = 0; i < PDO_Cnt; i++)
                {
                    for (j = 0; j < 8; j++)
                    {
                        Data[j] = 0;
                    }

                    ret = PISOCANCPM.CPM_GetPDOLastData(BOARD, PORT, Id_List[i], out IsNew, out DLen, Data, BlockMode);
                    if ((ret == 0) || (ret == PISOCANCPM._15_CPM_DataEmpty))
                    {
                        Temp_Str = "TxPDO " + Id_List[i].ToString("X") + " : ";

                        for (j = 0; j < 8; j++)
                        {
                            Temp_Str = Temp_Str + Data[j].ToString("X2") + " ";
                        }

                        List_EntryData.Items.Add(Temp_Str);
                    }
                }
            }
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            UpdateList();
        }
    }
}